/*
    SPDX-FileCopyrightText: 2024 Ralf Habacker ralf.habacker @freenet.de

    This file is part of libalkimia.

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#ifndef ALKNEWSTUFFSTORE_H
#define ALKNEWSTUFFSTORE_H

#include <alkimia/alk_export.h>

class ALK_EXPORT AlkNewStuffStore {
public:
    static const char *alkimiaOnlineQuotes;
    static const char *packageBaseUrl;
    static const char *packageEditSuffix;
    static const char *newPackageUrl;
};

#endif // ALKNEWSTUFFSTORE_H
